<?php
/*
 * A payment plugin called "ipay". This is the main file of the plugin.
 */

// You need to extend from the hikashopPaymentPlugin class which already define lots of functions in order to simplify your work
class plgHikashoppaymentIpay extends hikashopPaymentPlugin
{
	//List of the plugin's accepted currencies. The plugin won't appear on the checkout if the current currency is not in that list. You can remove that attribute if you want your payment plugin to display for all the currencies
	// var $accepted_currencies = array( "EUR", "KES", "USD" );
	var $accepted_currencies = array("KES", "USD" );
	
	// Multiple plugin configurations. It should usually be set to true
	var $multiple = true;
	
	//Payment plugin name (the name of the PHP file)
	var $name = 'ipay';

	const ADDRESS_TYPE = 'billing_address';

	// This array contains the specific configuration needed (Back end > payment plugin edition), depending of the plugin requirements.
	// They will vary based on your needs for the integration with your payment gateway.
	// The first parameter is the name of the field. In upper case for a translation key.
	// The available types (second parameter) are: input (an input field), html (when you want to display some custom HTML to the shop owner), textarea (when you want the shop owner to write a bit more than in an input field), big-textarea (when you want the shop owner to write a lot more than in an input field), boolean (for a yes/no choice), checkbox (for checkbox selection), list (for dropdown selection) , orderstatus (to be able to select between the available order statuses)
	// The third parameter is the default value.
	var $pluginConfig = array(

		'vendorid' => array("Vendor ID",'input'),
		// User's password on the payment platform
		'hashkey' => array("Hash / Security Key",'input'),
		// To allow (or not) notifications from the payment platform. The plugin can only work if notifications are allowed
		'notification' => array('ALLOW_NOTIFICATIONS_FROM_X', 'boolean','0'),
		// Write some things on the debug file
		'testmode' => array('Sandbox Mode', 'boolean','0'),
		// The URL where the user is redirected after a fail during the payment process
		'cancel_url' => array('CANCEL_URL_DEFINE','html',''),
		// The URL where the payment platform the user about the payment (fail or success)
		'notify_url' => array('NOTIFY_URL_DEFINE','html',''),
		// Invalid status for order in case of problem during the payment process
		'invalid_status' => array('INVALID_STATUS', 'orderstatus'),
		// Valid status for order if the payment has been done well
		'verified_status' => array('VERIFIED_STATUS', 'orderstatus')
	);

	/**
	 * The constructor is optional if you don't need to initialize some parameters of some fields of the configuration and not that it can also be done in the getPaymentDefaultValues function as you will see later on
	 */
	function __construct(&$subject, $config)
	{
		$this->pluginConfig['notification'][0] =  JText::sprintf('ALLOW_NOTIFICATIONS_FROM_X','iPay');
		
		// This is the cancel URL of HikaShop that should be given to the payment gateway so that it can redirect to it when the user cancel the payment on the payment gateway page. That URL will automatically cancel the order of the user and redirect him to the checkout so that he can choose another payment method
		$this->pluginConfig['cancel_url'][2] = HIKASHOP_LIVE."index.php?option=com_hikashop&ctrl=order&task=cancel_order";
		
		// This is the "thank you" or "return" URL of HikaShop that should be given to the payment gateway so that it can redirect to it when the payment of the user is valid. That URL will reinit some variables in the session like the cart and will then automatically redirect to the "return_url" parameter
		$this->pluginConfig['return_url'][2] = HIKASHOP_LIVE."index.php?option=com_hikashop&ctrl=checkout&task=after_end";
		
		// This is the "notification" URL of HikaShop that should be given to the payment gateway so that it can send a request to that URL in order to tell HikaShop that the payment has been done (sometimes the payment gateway doesn't do that and passes the information to the return URL, in which case you need to use that notification URL as return URL and redirect the user to the HikaShop return URL at the end of the onPaymentNotification function)
		$this->pluginConfig['notify_url'][2] = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=notify&amp;notif_payment='.$this->name.'&tmpl=component';
		
		return parent::__construct($subject, $config);
	}


	/**
	 * This function is called at the end of the checkout. That's the function which should display your payment gateway redirection form with the data from HikaShop
	 */
	function onAfterOrderConfirm(&$order, &$methods, $method_id)
	{
		// This is a mandatory line in order to initialize the attributes of the payment method
		parent::onAfterOrderConfirm($order,$methods,$method_id);

		// Here we can do some checks on the options of the payment method and make sure that every required parameter is set and otherwise display an error message to the user
		// The plugin can only work if those parameters are configured on the website's backend
		if (empty($this->payment_params->vendorid))
		{
			// Enqueued messages will appear to the user, as Joomla's error messages
			$this->app->enqueueMessage('You have to configure the vendor id for the iPay plugin payment first : check your plugin\'s parameters, on your website backend', 'error');
			return false;
		}
		elseif (empty($this->payment_params->hashkey))
		{
			$this->app->enqueueMessage('You have to configure a hashkey for the iPay plugin payment first : check your plugin\'s parameters, on your website backend', 'error');
			return false;
		}
		else
		{
			// Here, all the required parameters are valid, so we can proceed to the payment platform
			$user = JFactory::getUser();
			$address = $this->getAddress($order);

			// Debug mode activated or not
			if($this->payment_params->testmode) {
				$live = "0";
			} else {
				$live = "1";
			}
	        
	        $mm     = 1;
	        $mb     = 1;
	        $dc     = 1;
	        $cc     = 1;
	        
	        $oid    = $order->order_id;
	        $inv    = $order->order_id;
	        $ttl    = round($order->cart->full_total->prices[0]->price_value_with_tax, 2);
	        $tel    = $address->address_telephone;
	        $eml    = $order->customer->user_email;

	        /**
	         * incase of any dashes in the telephone number the code below removes them
	         * @var [string]
	         */
	        $tel    = str_replace("-", "", $tel);
	        $tel    = str_replace( array(' ', '<', '>', '&', '{', '}', '*', "+", '!', '@', '#', "$", '%', '^', '&'), "", $tel );
	        
	        $vid    = $this->payment_params->vendorid;
	        $curr   = $this->currency->currency_code;
	        
	        /**
	         * $p1, $p2, $p3, $p4  are optional fields. Allow sending & receiving your custom parameters
	         * Each option should not exceed 20 characters.
	         */

	        $p1     = '';
	        $p2     = '';
	        $p3     = ''; 
	        $p4     = '';
	        

	        /**
	         * [$callbk holds the callback URL]
	         */
	        $callbk = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=notify&notif_payment='.$this->name.'&tmpl=component&lang='.$this->locale.$this->url_itemid.'&orderID='.$order->order_id;
	        $linkbk = HIKASHOP_LIVE.'index.php';
	        
	        $cst    = 1;
	        $crl    = 0;

	        
	        /**
	         * [$hsh holds the merchant's secret key]
	         */
	        $hsh = $this->payment_params->hashkey;
	        
	        //The data string values
	        $datastring =$live.$oid.$inv.$ttl.$tel.$eml.$vid.$curr.$p1.$p2.$p3.$p4.$callbk.$cst.$crl;

	        //Setting the hashing algorithm to SHA1
	        $hashid = hash_hmac('sha1', $datastring, $hsh);

	        /*URLENCODE*/
	        $cbk = urlencode($callbk);
	        $lbk = urlencode($linkbk);

			/* Bongapoints Channel + Plugin ID */
			$bonga   = "1";
			$channel = "HikaShop";

	        $this->payment_params->url = 'https://payments.ipayafrica.com/v3/ke?live='.$live.'&oid='.$oid.'&inv='.$inv.'&ttl='.$ttl.'&tel='.$tel.'&eml='.$eml.'&vid='.$vid.'&p1='.$p1.'&p2='.$p2.'&p3='.$p3.'&p4='.$p4.'&cbk='.$cbk.'&cst='.$cst.'&crl='.$crl.'&curr='.$curr.'&hsh='.$hashid.'&bonga='.$bonga.'&vooma=1'.'&channel='.$channel;
			
			// Ending the checkout, ready to be redirect to the plateform payment final form
			// The showPage function will call the ipay_end.php file which will display the redirection form containing all the parameters for the payment platform
			return $this->showPage('end'); 
		}
	}

	/**
	 * To set the specific configuration (back end) default values (see $pluginConfig array)
	 */
	function getPaymentDefaultValues(&$element)
	{
		$element->payment_name = 'iPay';
		$element->payment_description = '<p>Pay with Mobile Money or Card Online</p>
										<p><img title="iPay channels" src="https://resources.elipa.co/iPay_10_payment_channels.png" alt="ipay channels" width="350" height="100" /></p>';
		$element->payment_images = 'ipay';
		$element->payment_params->address_type = 'billing';
		$element->payment_params->notification = 1;
		$element->payment_params->invalid_status = 'cancelled';
		$element->payment_params->verified_status = 'confirmed';
	}

	/**
	 * After submiting the plateform payment form, this is where the website will receive the response information from the payment gateway servers and then validate or not the order
	 */
	function onPaymentNotification(&$statuses)
	{	
		// We first create a filtered array from the parameters received
		$vars = array();
		
		$filter = JFilterInput::getInstance();
		
		// A loop to create an array $var with all the parameters sent by the payment gateway with a POST method, and loaded in the $_REQUEST
		foreach($_GET as $key => $value)
		{
			$key = $filter->clean($key);
			$value = hikaInput::get()->getString($key);
			$vars[$key] = $value;
		}

		$config = JFactory::getConfig();

		// The load the parameters of the plugin in $this->payment_params and the order data based on the order_id coming from the payment platform
		$order_id = (int)@$vars['id'];
		$dbOrder = $this->getOrder($order_id);

		// With the order, we can load the payment method, and thus all the payment parameters
		$this->loadPaymentParams($dbOrder);
      
		$this->loadOrderData($dbOrder);

		if(isset($vars["txncd"]))
		{
			$val           = $this->payment_params->vendorid;
			$val1          = $vars['id'];
			$val2          = $vars['ivm'];
			$val3          = $vars['qwh'];
			$val4          = $vars['afd'];
			$val5          = $vars['poi'];
			$val6          = $vars['uyt'];
			$val7          = $vars['ifd'];


			$ipnUrl        = "https://www.ipayafrica.com/ipn/?vendor=".$val."&id=".$val1."&ivm=".$val2."&qwh=".$val3."&afd=".$val4."&poi=".$val5."&uyt=".$val6."&ifd=".$val7;
			
          	if(!$this->payment_params->testmode){
              $ch = curl_init();
              curl_setopt($ch, CURLOPT_URL, $ipnUrl);
              curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
              curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
              $status = curl_exec($ch);
              curl_close($ch);
            } else {
              $status = 'aei7p7yrx4ae34';
            }

			$return_url = "";

			switch ($status) {
				case 'aei7p7yrx4ae34':
				case 'eq3i7p5yt7645e':
					$this->modifyOrder($order_id, $this->payment_params->verified_status, true, true);
					$return_url = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=after_end&order_id='.$order_id.$this->url_itemid;
					break;

				case 'cr5i3pgy9867e1':
				case 'dtfi4p7yty45wq':
				case 'bdi6p2yy76etrs':
				case 'fe2707etr5s4wq':
				default:
					$this->modifyOrder($order_id, $this->payment_params->invalid_status, true, true);
					$return_url = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=order&task=cancel_order&order_id='.$order_id.$this->url_itemid;
					break;
			}
		}

		$this->app->redirect($return_url);
      	return true;
	}

	private function getAddress($order)
	{
		$addressType = self::ADDRESS_TYPE;
		// below is equivalent to $order->cart->billing_address
		return $order->cart->$addressType;
	}
}